from collections import Counter
import logging
import unittest

LOGGER = logging.getLogger('poker-hands')


class Card(object):
    def __init__(self, value, suit):
        self.value = value
        self.suit = suit

    def __hash__(self):
        return hash(self.value)

    def __gt__(self, other):
        if not isinstance(other, Card):
            return False
        return self.value > other.value

    def __eq__(self, other):
        if not isinstance(other, Card):
            return False
        return self.value == other.value and self.suit == other.suit

    def __repr__(self):
        return str(self.value) + self.suit[0]


class InvalidHandException(Exception):
    pass


class Hand(object):

    rank = -1

    def __init__(self, cards):
        if len(cards) != 5:
            raise InvalidHandException("Card count was not 5!")
        self.cards = sorted(cards)
        self.values_counter = Counter([card.value for card in self.cards])

    def __gt__(self, other):

        LOGGER.debug("Comparing: %s and %s" % (self, other))
        if self.rank != other.rank:
            return self.rank > other.rank

        return self.compare_same(other)

    def __get_repeated_card(self, n):
        """
        get the n-th most repeated card. No promise is made about order of similarly repeated cards
        :param n: count rank of card to get (1, 2, 2, 3, 3)[1] -> 2 or 3, (1, 2, 2, 2)[0] -> 2 ...
        :return: nth most repeated card
        """
        return self.values_counter.most_common(5)[n]

    def compare_cards_high_to_low(self, other):
        card, other_card = None, None
        for card, other_card in zip(self.cards, other.cards):
            if card.value != other_card.value:
                break

        return card.value > other_card.value

    def compare_same(self, other):
        raise NotImplementedError("Can not compare an abstract hand!")

    @staticmethod
    def get_hand(cards):

        classification_candidates = [
            StraightFlush,
            FourOfAKind,
            FullHouse,
            Flush,
            Straight,
            ThreeOfAKind,
            TwoPairs,
            Pair,
        ]
        for candidate in classification_candidates:
            if candidate.is_valid(cards):
                LOGGER.debug("%s: %s" % (candidate.__name__, str(cards)))
                return candidate(cards)

        # nothing identified, play using highest cards
        return HighCard(cards)

    @staticmethod
    def is_valid(cards):
        raise NotImplemented("is_valid not implemented for abstract hand!")

    def __str__(self):
        return "%s - %s" % (self.__class__, ','.join([str(card.value) + card.suit[0] for card in self.cards]))


class StraightFlush(Hand):

    rank = 8

    def __init__(self, cards):
        super(StraightFlush, self).__init__(cards)

    @staticmethod
    def is_valid(cards):
        return Flush.is_valid(cards) and Straight.is_valid(cards)

    def compare_same(self, other):
        return self.compare_cards_high_to_low(other)


class FourOfAKind(Hand):

    rank = 7

    def __init__(self, cards):
        super(FourOfAKind, self).__init__(cards)

    @staticmethod
    def is_valid(cards):
        counter = Counter([card.value for card in cards])
        return max(counter.values()) == 4

    def __repeated_card4(self):
        return self.values_counter.most_common(1)[0]

    def compare_same(self, other):
        return self.__repeated_card4() > other.__repeated_card4()


class FullHouse(Hand):

    rank = 6

    def __init__(self, cards):
        super(FullHouse, self).__init__(cards)

    @staticmethod
    def is_valid(cards):
        counter = Counter([card.value for card in cards])
        return 2 in counter.values() and 3 in counter.values()

    def __repeated_card3(self):
        return self.values_counter.most_common(1)[0]

    def compare_same(self, other):
        return self.__repeated_card3() > other.__repeated_card3()


class Flush(Hand):

    rank = 5

    def __init__(self, cards):
        super(Flush, self).__init__(cards)

    @staticmethod
    def is_valid(cards):
        return all(card.suit == cards[0].suit for card in cards)

    def compare_same(self, other):
        return self.compare_cards_high_to_low(other)


class Straight(Hand):

    rank = 4

    def __init__(self, cards):
        super(Straight, self).__init__(cards)

    @staticmethod
    def is_valid(cards):
        return all(card.value == v for card, v in zip(cards, range(cards[0].value, cards[0].value + 5)))

    def compare_same(self, other):
        self.compare_cards_high_to_low(other)


class ThreeOfAKind(Hand):

    rank = 3

    def __init__(self, cards):
        super(ThreeOfAKind, self).__init__(cards)

    @staticmethod
    def is_valid(cards):
        counter = Counter([card.value for card in cards])
        return max(counter.values()) == 3

    def __repeated_card3(self):
        return self.values_counter.most_common(1)[0]

    def compare_same(self, other):
        return self.__repeated_card3() > other.__repeated_card3()


class TwoPairs(Hand):

    rank = 2

    def __init__(self, cards):
        super(TwoPairs, self).__init__(cards)

    @staticmethod
    def is_valid(cards):
        counter = Counter([card.value for card in cards])
        return sorted(counter.values())[-2:] == [2, 2]

    def __repeated_cards2_2(self):
        return self.values_counter.most_common(2)

    def compare_same(self, other):
        high_pair, low_pair = self.__repeated_cards2_2()
        other_high_pair, other_low_pair = other.__repeated_cards2_2()
        if high_pair != other_high_pair:
            return high_pair > other_high_pair
        if low_pair != other_low_pair:
            return low_pair > other_low_pair

        return self.compare_cards_high_to_low


class Pair(Hand):

    rank = 1

    def __init__(self, cards):
        super(Pair, self).__init__(cards)

    @staticmethod
    def is_valid(cards):
        counter = Counter([card.value for card in cards])
        return max(counter.values()) == 2

    def __repeated_cards2(self):
        return self.values_counter.most_common(1)[0]

    def compare_same(self, other):
        if self.__repeated_cards2() != other.__repeated_cards2():
            return self.__repeated_cards2() > other.__repeated_cards2()

        return self.compare_cards_high_to_low(other)


class HighCard(Hand):

    rank = 0

    def __init__(self, cards):
        super(HighCard, self).__init__(cards)

    @staticmethod
    def is_valid(cards):
        return

    def compare_same(self, other):
        return self.compare_cards_high_to_low(other)


class TestHands(unittest.TestCase):

    def test_detection(self):
        straight_flush = Hand.get_hand([Card(value, 'Hearts') for value in range(5)])
        straight = Hand.get_hand([Card(value, 'Hearts') for value in range(4)] + [Card(4, 'Clubs')])
        assert max(straight, straight_flush) == straight_flush



from minipoker.logic.hands import *
from minipoker.logic.poker import *
from minipoker.logic.players import *
import unittest


class TestPokerGame(unittest.TestCase):
    def setUp(self):
        self.game = Poker([HumanPlayer("Player" + str(i), 100) for i in range(3)])

    def test_active_players_simple(self):
        self.game.play()
        assert self.game.current_round.active_players == self.game.players


class TestHands(unittest.TestCase):
    def test_straight_flush(self):
        assert type(Hand.get_hand([Card(v, 'SUIT') for v in [1, 2, 3, 4, 5]])) is StraightFlush
        assert type(Hand.get_hand([Card(v, 'SUIT') for v in [1, 2, 3, 5, 6]])) is not StraightFlush

    def test_four_of_a_kind(self):
        assert type(
            Hand.get_hand([Card(v, s) for v, s in [(1, 'h'), (1, 'd'), (1, 'c'), (1, 's'), (5, 'c')]])) is FourOfAKind
        assert type(Hand.get_hand(
            [Card(v, s) for v, s in [(1, 'h'), (1, 'd'), (1, 'c'), (2, 's'), (5, 'c')]])) is not FourOfAKind

    def test_full_house(self):
        assert type(
            Hand.get_hand([Card(v, s) for v, s in [(1, 'h'), (1, 'd'), (1, 'c'), (5, 's'), (5, 'c')]])) is FullHouse
        assert type(
            Hand.get_hand([Card(v, s) for v, s in [(1, 'h'), (1, 'd'), (1, 'c'), (2, 's'), (5, 'c')]])) is not FullHouse

    def test_flush(self):
        assert type(Hand.get_hand([Card(v, s) for v, s in [(1, 'd'), (2, 'd'), (5, 'd'), (6, 'd'), (7, 'd')]])) is Flush
        assert type(
            Hand.get_hand([Card(v, s) for v, s in [(1, 'h'), (1, 'd'), (1, 'c'), (2, 's'), (5, 'c')]])) is not Flush

    def test_straight(self):
        assert type(
            Hand.get_hand([Card(v, s) for v, s in [(1, 'd'), (2, 'd'), (3, 'd'), (4, 'c'), (5, 'd')]])) is Straight
        assert type(
            Hand.get_hand([Card(v, s) for v, s in [(1, 'h'), (1, 'd'), (1, 'c'), (2, 's'), (5, 'c')]])) is not Straight

    def test_three_of_a_kind(self):
        assert type(
            Hand.get_hand([Card(v, s) for v, s in [(1, 'd'), (1, 'd'), (1, 'd'), (4, 'c'), (5, 'd')]])) is ThreeOfAKind
        assert type(Hand.get_hand(
            [Card(v, s) for v, s in [(1, 'h'), (1, 'd'), (2, 'c'), (2, 's'), (5, 'c')]])) is not ThreeOfAKind

    def test_two_pairs(self):
        assert type(
            Hand.get_hand([Card(v, s) for v, s in [(1, 'h'), (1, 'd'), (2, 'c'), (2, 's'), (5, 'c')]])) is TwoPairs
        assert type(
            Hand.get_hand([Card(v, s) for v, s in [(1, 'd'), (1, 'd'), (1, 'd'), (4, 'c'), (5, 'd')]])) is not TwoPairs

    def test_one_pair(self):
        assert type(Hand.get_hand([Card(v, s) for v, s in [(1, 'h'), (10, 'd'), (2, 'c'), (2, 's'), (5, 'c')]])) is Pair
        assert type(
            Hand.get_hand([Card(v, s) for v, s in [(1, 'd'), (1, 'd'), (1, 'd'), (4, 'c'), (5, 'd')]])) is not Pair



import tkinter as tk
import queue
import threading
import logging

from minipoker.logic import poker as ppoker, players

FORMAT = '%(name)s - %(message)s'
logging.basicConfig(format=FORMAT, level=logging.DEBUG)
LOGGER = logging.getLogger("poker-gui")

event_queue = queue.Queue()
gui_to_logic_queue = queue.Queue()


class MESSAGES:
    REFRESH = "refresh"


class Menu(tk.Frame):
    def __init__(self, master):
        super(Menu, self).__init__(master, height=500, width=200)
        self.master = master
        players_selection_var = tk.IntVar(self)
        players_selection_var.set(2)
        players_menu = tk.OptionMenu(self, players_selection_var, 2, 3, 4, 5, 6, 7, 8)
        player_count_label = tk.Label(self)
        player_count_label['text'] = 'number of players'
        player_count_label.grid(row=0, column=0)
        players_menu.grid(row=0, column=1)

        cash_selection_var = tk.IntVar(self)
        cash_selection_var.set(100)
        cash_menu = tk.OptionMenu(self, cash_selection_var, 100, 500, 1000)
        starting_cash_label = tk.Label(self)
        starting_cash_label['text'] = 'starting cash'
        starting_cash_label.grid(row=1, column=0)
        cash_menu.grid(row=1, column=1)

        start_button = tk.Button(self,
                                 text='Start!',
                                 command=lambda: self.make_game(master, players_selection_var.get(),
                                                                cash_selection_var.get()))
        start_button.grid(row=3, column=0)

        self.pack()

    def make_game(self, master, player_count, cash):
        for widget in self.grid_slaves():
            widget.grid_forget()
        _players = [GUIHumanPlayer("Player %d" % i, cash, master) for i in range(player_count)]
        return Game(self, ppoker.Poker(_players))


ENABLE = 'active'
DISABLE = 'disabled'


class CallButton(tk.Button):
    def __init__(self, master):
        super(CallButton, self).__init__(master, text="call NA")
        self.action = players.Call

    def disable(self):
        self['state'] = DISABLE

    def enable(self):
        self['state'] = ENABLE

    def refresh(self, player, _round):
        if not player or not _round or not self.action.is_valid(player, _round):
            self.disable()
        else:
            self['text'] = 'Call %d' % _round.pot.amount_to_call(player)
            self['command'] = lambda: player.queue.put(self.action(player, _round))
            self.enable()


class CheckButton(tk.Button):
    def __init__(self, master):
        super(CheckButton, self).__init__(master, text="check NA")
        self.action = players.Check

    def disable(self):
        self['state'] = DISABLE

    def enable(self):
        self['state'] = ENABLE

    def refresh(self, player, _round):
        if not player or not _round or not self.action.is_valid(player, _round):
            self.disable()
        else:
            self['text'] = 'Check'
            self['command'] = lambda: player.queue.put(self.action(player, _round))
            self.enable()


class BetButton(tk.OptionMenu):
    def __init__(self, master):
        self.bet_var = tk.StringVar(master)
        self.bet_var.set("Bet NA")
        super(BetButton, self).__init__(master, self.bet_var, "Bet NA")
        self.action = players.Bet

    def disable(self):
        self['state'] = DISABLE

    def enable(self):
        self['state'] = ENABLE

    def refresh(self, player, _round):
        self['menu'].delete(0, tk.END)
        if not player or not _round or not self.action.is_valid(player, _round):
            self.disable()
        else:
            self.bet_var.set("Bet ...")
            minimum, maximum = _round.pot.minimum_to_bet(player), player.money
            step = max((maximum - minimum) // 10, 1)
            LOGGER.info("minimum: %d, maximum: %d, step: %d" % (minimum, maximum, step))

            def bet(_amount):
                return lambda: player.queue.put(players.Bet(player, _round, _amount))

            # add 10 betting options
            for amount in range(minimum, maximum, step):
                self['menu'].add_command(label="Bet %d" % amount,
                                         command=bet(amount))
            # add all-in option
            self['menu'].add_command(label="All In! (%d)" % maximum, command=bet(maximum))
            self.enable()


class FoldButton(tk.Button):
    def __init__(self, master):
        super(FoldButton, self).__init__(master, text="fold NA")
        self.action = players.Fold

    def disable(self):
        self['state'] = DISABLE

    def enable(self):
        self['state'] = ENABLE

    def refresh(self, player, _round):
        if not player or not _round or not self.action.is_valid(player, _round):
            self.disable()
        else:
            self['text'] = 'Fold'
            self['command'] = lambda: player.queue.put(self.action(player, _round))
            self.enable()


class GUIPlayer(tk.Frame):
    def __init__(self, master, player, moves):
        super(GUIPlayer, self).__init__(master)
        player.player_frame = self
        self.player = player
        self.moves = moves
        self.master = master
        self.row = player.id + 1
        self.name_label = tk.Label(master, text=player.name)
        self.cash_label = tk.Label(master, text=player.money)
        self.name_label.grid(row=self.row, column=0)
        self.cash_label.grid(row=self.row, column=1)

        self.pocket1 = tk.Label(master, text='na')
        self.pocket2 = tk.Label(master, text='na')
        self.pocket1.grid(row=self.row, column=2)
        self.pocket2.grid(row=self.row, column=3)

        self.pot = tk.Label(master, text='na')
        self.pot.grid(row=self.row, column=4)

        self.move_buttons = {
            "check": CheckButton(master),
            "bet": BetButton(master),
            "fold": FoldButton(master),
            "call": CallButton(master),
        }
        # place action buttons
        for offset, move in enumerate(self.move_buttons.values()):
            move.grid(row=self.row, column=5 + offset)
        self.refresh(None)

    def refresh(self, _round):
        if self.player.pocket:
            self.pocket1['text'] = self.player.pocket[0]
            self.pocket1['fg'] = self.player.pocket[0].color()
            self.pocket2['text'] = self.player.pocket[1]
            self.pocket2['fg'] = self.player.pocket[1].color()
        for move_button in self.move_buttons.values():
            move_button.refresh(self.player, _round)
        if _round:
            self.pot['text'] = _round.pot.player_bet(self.player)
            self.cash_label['text'] = "%d" % self.player.money

    def disable(self):
        for move_button in self.move_buttons.values():
            move_button['state'] = DISABLE


class GUIHumanPlayer(players.BasePlayer):
    def __init__(self, name, money, gui_game):
        super(GUIHumanPlayer, self).__init__(name, money)
        self.gui_game = gui_game
        LOGGER.debug("%s interacting" % self.name)
        self.queue = queue.Queue()
        self.player_frame = None
        self.gui = None

    def interact(self, _round):
        LOGGER.debug("refreshing gui")

        # refresh gui
        event_queue.put((MESSAGES.REFRESH, _round,))

        # get move
        return self.queue.get()

    def get_amount(self, _min, _max):
        self.queue.get()

    @staticmethod
    def gui_get_amount(_min, _max):
        while True:
            amount_selected_event = threading.Event()
            popup = tk.Toplevel()
            title = tk.Label(popup, text="How much [%d - %d]?" % (_min, _max), height=0, width=100)
            title.pack()

            amount_var = tk.StringVar()
            amount_var.set("%d" % _min)
            entry = tk.Entry(popup, textvariable=amount_var)
            entry.pack()

            confirm_amount = tk.Button(popup, text="ok", command=lambda: amount_selected_event.set())
            confirm_amount.pack()

            amount_selected_event.wait()
            if not amount_var.get().isdigit():
                print("amount not a number")
                continue
            amount = int(amount_var.get())
            if not _min <= amount <= _max:
                print("amount not in range")
                continue
            return amount


class Game(object):
    def __init__(self, frame, logic):

        self.frame = frame

        self.game_logic = logic

        for player in self.game_logic.players:
            player.gui = self

        for offset, title in enumerate("Player,Money,Card 1,Card2,Invested".split(",")):
            tk.Label(self.frame, text=title).grid(row=0, column=offset)

        self.player_frames = {}
        for player in self.game_logic.players:
            player_frame = GUIPlayer(self.frame, player, [])
            player_frame.grid(row=player.id + 1)
            self.player_frames[player] = player_frame

        tk.Label(self.frame, text="Community Cards:").grid(row=len(self.game_logic.players) + 1)

        self.community_cards = [tk.Label(self.frame, text='') for _ in range(5)]
        for offset, community_card in enumerate(self.community_cards):
            community_card.grid(row=len(self.game_logic.players) + 2, column=offset)

        # start game only after gui initialized
        self.game_thread = threading.Thread(target=self.game_logic.play)
        self.game_thread.start()
        self.frame.after(10, func=self.process_event_queue)
        self.frame.pack()

    @staticmethod
    def get_amount(_min, _max):
        print("get amount %d - %d" % (_min, _max))

    def process_event_queue(self):
        # this is needed to let UI thread update widgets
        # (tkinter does not support multi threading)
        try:
            # LOGGER.debug("processing events") # too much output
            message = event_queue.get(block=False)
            if message[0] == MESSAGES.REFRESH:
                _round = message[1]
                for player_frame in self.player_frames.values():
                    player = player_frame.player
                    if _round:
                        LOGGER.debug("Populating: %s" % player.name)
                        player_frame.refresh(_round)
                    if player is not _round.betting_player:
                        player_frame.disable()
                    LOGGER.debug("refreshing community cards %s" % str(_round.community_cards))
                    for label, card in zip(self.community_cards, _round.community_cards + [''] * 5):
                        label['text'] = card
                        if card:
                            LOGGER.debug("setting color to: %s" % card.color())
                            label['fg'] = card.color()
            self.frame.pack()
        except queue.Empty:
            pass
        self.frame.after(10, func=self.process_event_queue)


def main():
    root = tk.Tk()
    Menu(root)
    root.mainloop()



from itertools import combinations
import os
import logging

from minipoker.logic.hands import Hand

LOGGER = logging.getLogger('poker-players')


class Action(object):

    def __init__(self, player, round_):
        self.player = player
        self.round = round_

    @staticmethod
    def is_valid(player, round_):
        raise NotImplementedError("Can not validate abstract Action")

    def apply(self):
        raise NotImplementedError("Can not apply abstract Action")


class AmountableAction(Action):

    def __init__(self, player, round_, amount):
        super(AmountableAction, self).__init__(player, round_)
        self.amount = amount

    @staticmethod
    def is_valid(player, round_):
        raise NotImplementedError("Can not validate abstract Action")

    def apply(self):
        raise NotImplementedError("Can not apply abstract Action")


class Fold(Action):

    name = "Fold"

    @staticmethod
    def is_valid(player, round_):
        return player not in round_.folded_players

    def apply(self):
        self.round.folded_players.append(self.player)

    def __str__(self):
        return "Player %s folds" % self.player


class Check(Action):

    name = "Check"

    @staticmethod
    def is_valid(player, round_):
        pot = round_.pot
        return pot.player_bet(player) == pot.current_bet

    def apply(self):
        self.player.checked = True

    def __str__(self):
        return "Player %s checks" % self.player


class Call(AmountableAction):

    name = "Call"

    def __init__(self, player, round_):
        super(Call, self).__init__(player, round_, round_.pot.amount_to_call(player))

    @staticmethod
    def is_valid(player, round_):
        return player.money >= round_.pot.amount_to_call(player) > 0

    def apply(self):
        self.player.bet(self.amount, self.round)

    def __str__(self):
        return "Player %s calls %d" % (self.player, self.amount)


class Bet(AmountableAction):

    name = "Bet"

    def __init__(self, player, round_, amount=None):
        # TODO: change bet_max to actual max with repect to round
        bet_min, bet_max = round_.pot.minimum_to_bet(player), player.money
        LOGGER.debug("Setting bet limits to %d-%d" % (bet_min, bet_max))
        # choose action amount (example: bet, 50)
        while (amount is None) or (bet_min > amount or amount > bet_max):
            try:
                amount = int(player.get_amount(bet_min, bet_max))
            except ValueError:
                print("Value has to be between 0 and ")
        LOGGER.debug("Player %s bet: %d" % (player.name, amount))

        super(Bet, self).__init__(player, round_, amount)

    @staticmethod
    def is_valid(player, round_):
        return player.money >= round_.pot.minimum_to_bet(player) > 0

    def apply(self):
        self.player.money -= self.amount
        self.round.bet(self.player, self.amount)


class NotEnoughMoneyException(Exception):
    pass


class BasePlayer(object):

    ids = 0

    def __init__(self, name, starting_money):
        self.id = BasePlayer.ids
        BasePlayer.ids += 1
        self.name = name
        self.pocket = None
        self.money = starting_money
        self.first_bet = False
        self.checked = False

    def __str__(self):
        return "[name: %s, money: %s]" % (self.name, self.money)

    def set_pocket(self, card1, card2):
        self.pocket = [card1, card2]

    def bet(self, amount, round_):
        if amount > self.money:
            raise NotEnoughMoneyException(
                "Player " + self.name + " does not have enough money (" + self.money + "/" + amount + ")"
            )
        self.money -= amount
        round_.bet(self, amount)

    def force_bet(self, amount, round_):
        """
        Force a bet from the player, even if the amount exceeds the player's funds
        :param amount: the amount to force on the player
        :return:
                amount of money actually taken
        """
        LOGGER.info("Forcing %s to bet %d" % (self.name, amount))
        amount = min(amount, self.money)
        self.bet(amount, round_)
        return amount

    def interact(self, round_):
        raise NotImplementedError("Interact is not implemented on the Player's base class")

    def get_amount(self, _min, _max):
        raise NotImplementedError("get_amount is not implemented on the Player's base class")

    def is_folded(self, round_):
        return round_.is_folded(self)

    def is_betting(self, round_):
        return len(round_.active_players) > 1 and self.money > 0 and self.first_bet or (
            not self.is_folded(round_) and round_.pot.amount_to_call(self) > 0)

    def possible_hands(self, community_cards):
        return [Hand.get_hand(cards) for cards in combinations(community_cards + self.pocket, r=5)]

    def best_hand(self, community_cards):
        # TODO: add aces multiple value (1, 14)
        # print("Community Cards: %s" % community_cards)
        # print("Pocket cards: %s" % self.pocket)
        # print("possible hands:\n%s" % '\n'.join(map(str, self.possible_hands(community_cards))))
        best_hand = max(self.possible_hands(community_cards))
        return best_hand

    def available_actions(self, _round):
        LOGGER.debug("fetching available moves")
        if not _round:
            LOGGER.debug("0 moves for false round")
            return []
        LOGGER.debug("round is ok, fetching moves")
        return list(filter(lambda x: x.is_valid(self, _round), [Check, Call, Bet, Fold]))

    def choose_action_message(self, round_):
        return os.linesep.join([
            "%s, your cards are %s please choose an action:" % (self.name, self.pocket)
        ] + ['. '.join(
            [str(index), action_name.__name__]
        ) for index, action_name in enumerate(self.available_actions(round_))] + [
            "Community cards: %s" % round_.community_cards
        ] + ["%s bet %d" % (player, round_.pot.player_bet(player)) for player in round_.players])


class HumanPlayer(BasePlayer):

    def interact(self, round_):

        action = None
        while action is None:
            try:
                available_actions = self.available_actions(round_)
                action = available_actions[int(input(self.choose_action_message(round_)))]
                return action(self, round_)
            except ValueError:
                pass
            except IndexError:
                pass

    def get_amount(self, _min, _max):
        return input("How much [%d - %d]?" % (_min, _max))



from random import shuffle

SUITS = ("Hearts", "Diamonds", "Clubs", "Spades")
symbols = {"Spades": u'♠', "Hearts": u'♥', "Diamonds": u'♦', "Clubs": u'♣'}


class Card(object):
    def __init__(self, value, suit):
        self.value = value
        self.suit = suit

    def __gt__(self, other):
        if not isinstance(other, Card):
            return False
        return self.value > other.value

    def __eq__(self, other):
        if not isinstance(other, Card):
            return False
        return self.value == other.value and self.suit == other.suit

    def __repr__(self):
        return str(self.value) + self.suit[0]

    def __str__(self):
        return str(self.value) + symbols[self.suit]

    def color(self):
        return "red" if self.suit in ("Hearts", "Diamonds") else "black"


class Deck(object):
    def __init__(self):
        self.cards = [Card(value, suit) for value in range(1, 14) for suit in SUITS]
        # don't forget to shuffle
        self.shuffle()

    def shuffle(self):
        shuffle(self.cards)

    def draw(self, number=1):
        drawn, self.cards = self.cards.pop(0), self.cards[number:]
        return drawn



from setuptools import setup, find_packages
from os import path
here = path.abspath(path.dirname(__file__))
# Get the long description from the relevant file
setup(
    name='minipoker',
    version='0.0.2',
    description='Poker Game',
    long_description='Simple implementation for a poker game',
    url='https://github.com/reutsharabani/minipoker',
    # Author details
    author='Reut Sharabani',
    author_email='reut.sharabani@gmail.com',
    # Choose your license
    license='MIT',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: End Users/Desktop',
        'Topic :: Games/Entertainment :: Board Games',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3'
    ],
    # What does your project relate to?
    keywords='poker game',
    # You can just specify the packages manually here if your project is
    #  simple. Or you can use find_packages().
    packages=find_packages(exclude=['contrib', 'docs', 'tests*']),
    install_requires=['']
)


from random import choice
from collections import defaultdict
import logging
from minipoker.logic.deck import Deck

LOGGER = logging.getLogger('poker-main')


class Pot(object):
    def __init__(self, small_blind):
        self.bets = defaultdict(int)
        self.last_raise = small_blind * 2

    @property
    def current_bet(self):
        return max(self.bets.values())

    def player_bet(self, player):
        return self.bets[player]

    def amount_to_call(self, player):
        return self.current_bet - self.bets[player]

    def minimum_to_bet(self, player):
        LOGGER.debug("current bet: %d, last raise: %d, player_bet: %d" % (
            self.current_bet, self.last_raise, self.player_bet(player)
        ))
        return max(1, self.amount_to_call(player) + self.last_raise)

    def take_pot_for_player(self, player):
        player_bet = self.player_bet(player)

        winnings = 0
        # take amount from all bets
        for player, bet in self.bets.items():
            # see how much is possible to take from player
            possible_to_take = min(bet, player_bet)
            winnings += possible_to_take
            # remove as much as possible from player
            self.bets[player] -= possible_to_take

        return winnings

    def bet(self, player, amount):
        LOGGER.info("%s bets %d" % (player, amount))
        self.bets[player] += amount


class Round(object):
    def __init__(self, players, button_player, small_blind):
        self.players = players[:]
        self.folded_players = []
        self.small_blind = small_blind
        self.betting_player = None
        self.pot = None
        self.deck = Deck()
        self.community_cards = []
        self.button_player = button_player
        self.pot = Pot(self.small_blind)
        LOGGER.info("Dealing cards")
        for player in self.players:
            player.set_pocket(self.deck.draw(), self.deck.draw())

    def bet(self, player, amount):
        self.pot.bet(player, amount)

    def is_folded(self, player):
        return player in self.folded_players

    @property
    def active_players(self):
        return [player for player in self.players if player not in self.folded_players]

    def after(self, player):
        index = self.players.index(player)
        return self.players[(index + 1) % len(self.players)]

    def take_blinds(self):
        self.small_blind_player().force_bet(self.small_blind, self)
        self.big_blind_player().force_bet(2 * self.small_blind, self)
        return self.betting_player

    def small_blind_player(self):
        return self.after(self.button_player)

    def big_blind_player(self):
        return self.after(self.after(self.button_player))

    def winner(self):
        # check if only a single player is left
        return self.players[0] if len(self.players) - len(self.folded_players) == 1 else None

    def pre_flop_betting(self):
        if self.winner() is not None:
            return
        self.betting_player = self.after(self.big_blind_player())
        self.place_bets()

    def pre_turn_betting(self):
        if self.winner() is not None:
            return
        self.betting_player = self.after(self.button_player)
        self.place_bets()

    def pre_river_betting(self):
        if self.winner() is not None:
            return
        self.betting_player = self.after(self.button_player)
        self.place_bets()

    def final_betting(self):
        if self.winner() is not None:
            return
        self.betting_player = self.after(self.button_player)
        self.place_bets()

    def next_betting_player(self):
        start = self.betting_player
        candidate = self.after(start)
        while candidate is not start:
            if candidate.is_betting(self):
                LOGGER.info("Found betting player %s" % candidate)
                return candidate
            LOGGER.info("Skipping player %s" % candidate)
            candidate = self.after(candidate)
        return None

    def place_bets(self):
        # when no betting players are left
        # self.betting_player will be set to None

        # set player first_bet to True to indicate this player is yet to bet
        for player in self.active_players:
            player.first_bet = True

        if not self.betting_player.is_betting(self):
            LOGGER.info("Skipping player: %s" % str(self.betting_player))
            self.betting_player = self.next_betting_player()

        while self.betting_player is not None:
            LOGGER.info("player %s is choosing an action", self.betting_player.name)
            self.betting_player.first_bet = False
            action = self.betting_player.interact(self)
            LOGGER.info("%s chose Action: %s" % (self.betting_player.name, action.__class__.__name__))
            action.apply()
            self.betting_player = self.next_betting_player()

        LOGGER.info("Done betting for pre_flop_round")

    def play(self):

        LOGGER.info("Playing another round of poker")

        self.take_blinds()

        LOGGER.info("Playing first round (pre-flop)")
        self.pre_flop_betting()
        self.open_flop_cards()

        LOGGER.info("Playing first round (pre-turn)")
        self.pre_turn_betting()
        self.open_turn_cards()

        LOGGER.info("Playing first round (pre-river)")
        self.pre_river_betting()
        self.open_river_cards()
        self.final_betting()
        LOGGER.info("Round winners:")
        for winner_ in self.get_round_winners():
            LOGGER.info("%s" % winner_)
        return dict(self.finish_round())

    def open_card(self):
        self.community_cards.append(self.deck.draw())

    def open_flop_cards(self):
        self.open_card()
        self.open_card()
        self.open_card()

    def open_turn_cards(self):
        self.open_card()

    def open_river_cards(self):
        self.open_card()

    def finish_round(self):
        for winner_ in self.get_round_winners():
            winnings = self.pot.take_pot_for_player(winner_)
            LOGGER.info("Giving winnings (%d) to player: %s [%s]" % (
                winnings, winner_.name, winner_.best_hand(self.community_cards)))
            winner_.money += winnings
            yield winner_, winnings

    def get_round_winners(self):
        return sorted(
            set(self.players) - set(self.folded_players), key=lambda x: x.best_hand(self.community_cards), reverse=True
        )


class Poker(object):
    def __init__(self, players):

        # list of players who ran out of money
        self.finished_players = []

        # list of all players in game
        self.players = players

        self.button_player = choice(self.players)
        self.small_blind = 1
        self.rounds = []

        self.current_round = None

    def winner(self):
        """
        :return: game's winner, if one exists (None otherwise)
        """
        return self.players[0] if len(self.players) == 1 else None

    def after(self, player):
        wanted_player_index = (self.players.index(player) + 1) % len(self.players)
        return self.players[wanted_player_index]

    def advance_button_player(self):
        self.button_player = self.after(self.button_player)
        while self.button_player.money == 0:
            # keep searching for valid button player
            self.button_player = self.after(self.button_player)

    def play(self):
        LOGGER.debug("Starting game")
        while self.winner() is None:
            LOGGER.info("Playing round #%d" % (len(self.rounds) + 1))
            LOGGER.info("Players are:")
            for player in self.players:
                LOGGER.info("%s" % player)
            round_ = Round(self.players, self.button_player, self.small_blind)
            self.current_round = round_
            winnings = round_.play()
            self.rounds.append(round_)

            for player, winning in winnings.items():
                print("%s won %d" % (player, winning))
            # move button (before possibly removing button player)
            self.advance_button_player()

            # set up players
            for player in self.players:
                if player.money == 0:
                    LOGGER.info("Player %s finished the game" % str(player))
                    self.finished_players.append(player)
                    self.players.remove(player)
        return self.winner()
